// src/index.ts
import { createUnplugin } from "unplugin";
import { createFilter } from "@rollup/pluginutils";
import {
  REGEX_SETUP_SFC,
  REGEX_VUE_SFC,
  REGEX_VUE_SUB,
  detectVueVersion
} from "@vue-macros/common";
import { RollupResolve, setResolveTSFileIdImpl } from "@vue-macros/api";

// src/core/index.ts
import {
  DEFINE_EMIT as DEFINE_EMIT2,
  DEFINE_PROP as DEFINE_PROP2,
  MagicString,
  getTransformResult,
  parseSFC
} from "@vue-macros/common";

// src/core/define-emit.ts
import { DEFINE_EMIT, isCallOf, walkAST } from "@vue-macros/common";

// src/core/constants.ts
import { HELPER_PREFIX } from "@vue-macros/common";
var PROPS_VARIABLE_NAME = `${HELPER_PREFIX}props`;
var EMIT_VARIABLE_NAME = `${HELPER_PREFIX}emit`;

// src/core/define-emit.ts
function mountEmits(emits) {
  const isAllWithoutOptions = emits.every(({ validator }) => !validator);
  if (isAllWithoutOptions) {
    return `[${emits.map(({ name: name2 }) => JSON.stringify(name2)).join(", ")}]`;
  }
  return `{
      ${emits.map(
    ({ name: name2, validator }) => `${JSON.stringify(name2)}: ${validator || `null`}`
  ).join(", ")}
    }`;
}
function transformDefineEmit({ setupAst, offset, s }) {
  const emits = [];
  walkAST(setupAst, {
    enter(node) {
      if (isCallOf(node, DEFINE_EMIT)) {
        const [name2, validator] = node.arguments;
        if (name2.type !== "StringLiteral") {
          throw new Error(
            `The first argument of ${DEFINE_EMIT} must be a string literal.`
          );
        }
        emits.push({
          name: name2.value,
          validator: validator ? s.sliceNode(validator, { offset }) : void 0
        });
        s.overwriteNode(
          node,
          `(...args) => ${EMIT_VARIABLE_NAME}(${JSON.stringify(
            name2.value
          )}, ...args)`,
          { offset }
        );
      }
    }
  });
  if (emits.length > 0) {
    s.prependLeft(
      offset,
      `
const ${EMIT_VARIABLE_NAME} = defineEmits(${mountEmits(emits)})
`
    );
  }
}

// src/core/define-prop.ts
import {
  DEFINE_PROP,
  HELPER_PREFIX as HELPER_PREFIX2,
  importHelperFn,
  isCallOf as isCallOf2,
  walkAST as walkAST2
} from "@vue-macros/common";
import {
  inferRuntimeType,
  resolveTSReferencedType,
  toRuntimeTypeString
} from "@vue-macros/api";
async function mountProps({ id, scriptSetup, setupAst, isProduction }, props) {
  const isAllWithoutOptions = props.every(
    ({ definition, typeParameter }) => !definition && !typeParameter
  );
  if (isAllWithoutOptions) {
    return `[${props.map(({ name: name2 }) => JSON.stringify(name2)).join(", ")}]`;
  }
  let propsString = "{\n";
  for (const { name: name2, definition, typeParameter } of props) {
    propsString += `  ${JSON.stringify(name2)}: `;
    if (definition) {
      propsString += definition;
    } else if (typeParameter && !isProduction) {
      const resolved = await resolveTSReferencedType({
        scope: {
          filePath: id,
          content: scriptSetup.content,
          ast: setupAst.body
        },
        type: typeParameter
      });
      propsString += resolved ? `{ type: ${toRuntimeTypeString(await inferRuntimeType(resolved))} }` : "null";
    } else {
      propsString += "null";
    }
    propsString += ",\n";
  }
  propsString += "}";
  return propsString;
}
async function transformDefineProp(options) {
  const { setupAst, offset, s } = options;
  const props = [];
  walkAST2(setupAst, {
    enter(node) {
      var _a;
      if (isCallOf2(node, DEFINE_PROP)) {
        const [name2, definition] = node.arguments;
        if (name2.type !== "StringLiteral") {
          throw new Error(
            `The first argument of ${DEFINE_PROP} must be a string literal.`
          );
        }
        props.push({
          name: name2.value,
          definition: definition ? s.sliceNode(definition, { offset }) : void 0,
          typeParameter: (_a = node.typeParameters) == null ? void 0 : _a.params[0]
        });
        s.overwriteNode(
          node,
          `${HELPER_PREFIX2}computed(() => ${PROPS_VARIABLE_NAME}[${JSON.stringify(
            name2.value
          )}])`,
          { offset }
        );
      }
    }
  });
  if (props.length > 0) {
    importHelperFn(s, offset, "computed", "vue");
    s.prependLeft(
      offset,
      `
const ${PROPS_VARIABLE_NAME} = defineProps(${await mountProps(
        options,
        props
      )})
`
    );
  }
}

// src/core/index.ts
async function transformDefineSingle(code, id, isProduction) {
  const { scriptSetup, getSetupAst } = parseSFC(code, id);
  if (!scriptSetup)
    return;
  const offset = scriptSetup.loc.start.offset;
  const s = new MagicString(code);
  const setupAst = getSetupAst();
  const options = {
    id,
    s,
    offset,
    scriptSetup,
    setupAst,
    isProduction
  };
  if (code.includes(DEFINE_PROP2)) {
    await transformDefineProp(options);
  }
  if (code.includes(DEFINE_EMIT2)) {
    await transformDefineEmit(options);
  }
  return getTransformResult(s, id);
}

// src/index.ts
function resolveOption(options, framework) {
  const version = options.version || detectVueVersion();
  return {
    include: [REGEX_VUE_SFC, REGEX_SETUP_SFC].concat(
      version === 2 && framework === "webpack" ? REGEX_VUE_SUB : []
    ),
    isProduction: process.env.NODE_ENV === "production",
    ...options,
    version
  };
}
var name = "unplugin-vue-single-define";
var src_default = createUnplugin(
  (userOptions = {}, { framework }) => {
    const options = resolveOption(userOptions, framework);
    const filter = createFilter(options.include, options.exclude);
    const { resolve, handleHotUpdate } = RollupResolve();
    return {
      name,
      enforce: "pre",
      buildStart() {
        if (framework === "rollup" || framework === "vite") {
          setResolveTSFileIdImpl(resolve(this));
        }
      },
      transformInclude(id) {
        return filter(id);
      },
      transform(code, id) {
        try {
          return transformDefineSingle(code, id, options.isProduction);
        } catch (err) {
          this.error(`${name} ${err}`);
        }
      },
      vite: {
        configResolved(config) {
          options.isProduction = config.isProduction;
        },
        handleHotUpdate
      }
    };
  }
);

export {
  src_default
};
